//-*-C++-*- Header file for the pControl Class
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// This file is unreadable, because the comment-docifier is
// brainedead.  Try a "make srcdocs" to generate the
// HTML-documentation.
//
#ifndef _pControl_h_
#define _pControl_h_ 1
#include <qwidget.h>
#include <qchkbox.h>
#include <qlabel.h>
#include <qsize.h>
#include "PixButton.h"
#include "paneldefs.h"





/**
  This is a {\bf QWidget} containing buttons that control program
  operations.  There are the obvious buttons: " power" to exit the
  program, and "iconify" to iconify it.  There are some not-so-obvious
  ones, too.

  There is a button for ejecting a CD or loading things.  It can have
  either an "eject" icon on it, or a folder.  That way, the button can
  {\em truly} do double-duty and not confuse the end-user.

  There is a "lock" check button, and a lock-light.  When the lock
  button is checked, the lock-light changes color to indicate that the
  program is now "locked".  [What it means for the program to be
  "locked" really depends on implementation external to this widget,
  but it should disable the power button and the load/eject button at
  the very least.]

  There is also a check button for toggling the presence of the main
  window's title bar, permitting the user to change the look and feel
  of the main window on-the-fly.

  {\bf
  \begin{verbatim}
  
  ----------------------------------------------------------
  ----------------------------------------------------------
  \end{verbatim}
  }

  @memo Control panel object
  @see PixButton, PixData
  */
class pControl : public QWidget
{
	 //
	 Q_OBJECT

   public:
	 ///
	 bool hasTitlebar;
	 /**
	   State flags.  \Ref{isLocked} is set when \Ref{lockb} is
	   checked.  \Ref{hasTitlebar} is set when \Ref{titleb} is
	   checked.
	   @memo
	   */
	 bool isLocked;
	 ///
	 PixButton *iconifyb;
	 ///
	 PixButton *loadejectb;
	 /**
	   The \Ref{PixButton}s that control iconification of the entire
	   program, loading/ejecting a CD or audio playlist, and the
	   power button, repsectively.
	   @memo
	   */
	 PixButton *powerb;
	 /**
	   The lock button.  Changes the state of the lock light and the
	   value of the \Ref{isLocked} flag accordingly, emitting the
	   \Ref{lockThis} signal with the current state.  It should be
	   used to deactivate \Ref{powerb} and \Ref{loadejectb}.
	   @memo
	   */
	 QCheckBox *lockb;
	 /**
	   Changes the value of the \Ref{hasTitlebar} flag and emits the
	   \Ref{titlebarOn} signal with the current state.  It should be
	   used to remove or reinstate any title bar and menu on the main
	   program window.  That way, the program can have a sleek, thin
	   border to it and look less like a computer program and more
	   like an audio device.
	   @memo
	   */
	 QCheckBox *titleb;

   public:
	 /**
	   Class Constructor
	   @param parent The parent of this object
	   @param scale Determines the size of the \Re{PixButton}s in
	   this object.  It's a multiplicative factor, so {\em scale}=2
	   means double the default size, and so on.
	   @param useEject When true, tells {\bf pControl} that it should
	   use an "eject" icon on the load/eject button.  When false,
	   {\bf pControl} will use a folder icon.
	   @memo
	   */
	 pControl( QWidget *parent, float scale=1.0, bool useEject=true );

	 /**
	   Class Destructor
	   @memo
	   */
	 ~pControl( void );

	 /**
	   Moves the object, performing some internal bookkeeping at the
	   same time.
	   @memo
	   */
	 inline void moveAll( int x, int y );

	 /**
	   Overloaded version.
	   @memo
	   */
	 inline void moveAll( QPoint pos );

	 /**
	   @return The point in the x-direction where this widget ends.
	   @memo
	   */
	 inline QPoint xEndPt(void);

	 /**
	   @return The point in the y-direction where this widget ends.
	   @memo
	   */
	 inline QPoint yEndPt(void);

	 /**
	   Changes the color used by the lock-light to indicate a
	   "locked" state.
	   @memo
	   */
	 inline void setLockedColor( QColor& c );

	 /**
	   Changes the color used by the lock-light to indicate an
	   "unlocked" state.
	   @memo
	   */
	 inline void setUnlockedColor( QColor& c );

	 /**
	   Changes the palette of this object to {\em p}.
	   @memo
	   */
	 void changePalette( QPalette& p );

	 /**
	   Changes the font used on the button labels.
	   @memo
	   */
	 void changeFont( QFont& f );


	 //
   signals:
	 /** 
	   {\em Signal}

	   Emitted when the \Ref{lockb} is checked.
	   @memo [{\tt signal}]
	   */
	 void lockThis(bool);

	 /** 
	   {\em Signal}

	   Emitted when the \Ref{titleb} is checked.
	   @memo [{\tt signal}]
	   */
	 void titlebarOn(bool);

	 /**
	   {\em Signal}

	   Emitted when a mouse press-event occurs anywhere else on the
	   panel outside of one of the widgets.  Contains the current
	   mouse position, in global screen coordinates.
	   @memo [{\tt signal}] */
	 void pressed(QPoint);

	 /**
	   {\em Signal}

	   Emitted when a mouse move-event occurs anywhere else on the
	   panel outside of one of the widgets.  Contains the current
	   mouse position, in global screen coordinates.
   	   @memo [{\tt signal}] */
	 void moving(QPoint);


	 //
   private:
	 void createChildren(float bscale,bool useEjectPix);
	 void locateChildren(void);
	 void moveChildren(void);
	 void mousePressEvent( QMouseEvent *me );
	 void mouseMoveEvent( QMouseEvent *me );
	 //
	 QColor col_locked, col_unlocked;
	 QLabel *locklight;
	 QPoint cbw_p, ej_p, pwr_p, ico_p, loc_p, llt_p, tbt_p;
	 QSize cbw_s;
	 int lock_w;
	 // Only static const members can be initialized.  BUT - this
	 // member will be *identical* for all objects declared under
	 // this class because of the "static"!
	 static const int bsep=WIDGET_SEP;

   private slots:
	 void toggle_lock();
	 void toggle_titlebar();
};




//----------------------------------------------------------
//
// Inlined methods.
//
//----------------------------------------------------------




// Moves the entire Function widget, including the children.
//
inline void pControl::moveAll( int x, int y )
{
  cbw_p.setX( x );
  cbw_p.setY( y );
  move( cbw_p );
}


// Overloaded version of the other function.
//
inline void pControl::moveAll( QPoint pos )
{
  cbw_p = pos;
  move( cbw_p );
}


// Returns a point specifying where to put the next widget, in the
// x-direction.
//
inline QPoint pControl::xEndPt( void )
{
  return( QPoint(cbw_s.width(),0) + cbw_p );
}


// Returns a point specifying where to put the next widget, in the
// y-direction.
//
inline QPoint pControl::yEndPt( void )
{
  return( QPoint(0,cbw_s.height()) + cbw_p );
}


// Sets the colors on the lock light
//
inline void pControl::setLockedColor( QColor& c )
{
   col_locked = c;
   if( isLocked )
	 locklight->setBackgroundColor( col_locked );
   else
	 locklight->setBackgroundColor( col_unlocked );
}
//
inline void pControl::setUnlockedColor( QColor& c )
{
   col_unlocked = c;
   if( isLocked )
	 locklight->setBackgroundColor( col_locked );
   else
	 locklight->setBackgroundColor( col_unlocked );
}



#endif //_pControl_h_
//||||||||||||||||||||||||||||||||
//
// End
// 
